/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2013 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include "JNpp_common.hpp"

jfieldID cuComplex_x; // float
jfieldID cuComplex_y; // float

jfieldID cuDoubleComplex_x; // double
jfieldID cuDoubleComplex_y; // double

jfieldID NppiPoint_x; // int
jfieldID NppiPoint_y; // int

jfieldID NppiSize_width; // int
jfieldID NppiSize_height; // int

jfieldID NppiRect_x; // int
jfieldID NppiRect_y; // int
jfieldID NppiRect_width; // int
jfieldID NppiRect_height; // int


jfieldID Npp16sc_re; // short
jfieldID Npp16sc_im; // short

jfieldID Npp32sc_re; // int
jfieldID Npp32sc_im; // int

jfieldID Npp64sc_re; // long
jfieldID Npp64sc_im; // long

jfieldID Npp32fc_re; // float
jfieldID Npp32fc_im; // float

jfieldID Npp64fc_re; // double
jfieldID Npp64fc_im; // double


// Class and method ID for TypedPointer and its constructor
jclass TypedPointer_Class;
jmethodID TypedPointer_Constructor;

/**
 * Creates a global reference to the class with the given name and
 * stores it in the given jclass argument, and stores the no-args
 * constructor ID for this class in the given jmethodID.
 * Returns whether this initialization succeeded.
 */
bool init(JNIEnv *env, const char *className, jclass &globalCls, jmethodID &constructor)
{
    jclass cls = NULL;
    if (!init(env, cls, className)) return false;
    if (!init(env, cls, constructor, "<init>", "()V")) return false;

    globalCls = (jclass)env->NewGlobalRef(cls);
    if (globalCls == NULL)
    {
        Logger::log(LOG_ERROR, "Failed to create reference to class %s\n", className);
        return false;
    }
    return true;
}

/**
 * Called when the library is loaded. Will initialize all
 * required field and method IDs
 */
JNIEXPORT jint JNICALL JNI_OnLoad(JavaVM *jvm, void *reserved)
{
    JNIEnv *env = NULL;
    if (jvm->GetEnv((void **)&env, JNI_VERSION_1_4))
    {
        return JNI_ERR;
    }

    Logger::log(LOG_TRACE, "Initializing JNpp\n");

    jclass cls = NULL;

    // Initialize the JNIUtils and PointerUtils
    if (initJNIUtils(env) == JNI_ERR) return JNI_ERR;
    if (initPointerUtils(env) == JNI_ERR) return JNI_ERR;

    // Obtain the constructor for the TypedPointer class
    if (!init(env, "jcuda/jnpp/TypedPointer", TypedPointer_Class, TypedPointer_Constructor)) return JNI_ERR;

    // Obtain the fieldIDs for cuComplex#x and cuComplex#y
    if (!init(env, cls, "jcuda/cuComplex")) return JNI_ERR;
    if (!init(env, cls, cuComplex_x, "x", "F")) return JNI_ERR;
    if (!init(env, cls, cuComplex_y, "y", "F")) return JNI_ERR;

    // Obtain the fieldIDs for cuDoubleComplex#x and cuDoubleComplex#y
    if (!init(env, cls, "jcuda/cuDoubleComplex")) return JNI_ERR;
    if (!init(env, cls, cuDoubleComplex_x, "x", "D")) return JNI_ERR;
    if (!init(env, cls, cuDoubleComplex_y, "y", "D")) return JNI_ERR;

    // Obtain the fieldIDs for NppiPoint
    if (!init(env, cls, "jcuda/jnpp/NppiPoint")) return JNI_ERR;
    if (!init(env, cls, NppiPoint_x, "x", "I")) return JNI_ERR;
    if (!init(env, cls, NppiPoint_y, "y", "I")) return JNI_ERR;

    // Obtain the fieldIDs for NppiSize
    if (!init(env, cls, "jcuda/jnpp/NppiSize")) return JNI_ERR;
    if (!init(env, cls, NppiSize_width,  "width",  "I")) return JNI_ERR;
    if (!init(env, cls, NppiSize_height, "height", "I")) return JNI_ERR;

    // Obtain the fieldIDs for NppiRect
    if (!init(env, cls, "jcuda/jnpp/NppiRect")) return JNI_ERR;
    if (!init(env, cls, NppiRect_x,      "x",      "I")) return JNI_ERR;
    if (!init(env, cls, NppiRect_y,      "y",      "I")) return JNI_ERR;
    if (!init(env, cls, NppiRect_width,  "width",  "I")) return JNI_ERR;
    if (!init(env, cls, NppiRect_height, "height", "I")) return JNI_ERR;

    // Obtain the fieldIDs for Npp16sc
    if (!init(env, cls, "jcuda/jnpp/types/Npp16sc")) return JNI_ERR;
    if (!init(env, cls, Npp16sc_re, "re", "S")) return JNI_ERR;
    if (!init(env, cls, Npp16sc_im, "im", "S")) return JNI_ERR;

    // Obtain the fieldIDs for Npp32sc
    if (!init(env, cls, "jcuda/jnpp/types/Npp32sc")) return JNI_ERR;
    if (!init(env, cls, Npp32sc_re, "re", "I")) return JNI_ERR;
    if (!init(env, cls, Npp32sc_im, "im", "I")) return JNI_ERR;

    // Obtain the fieldIDs for Npp64sc
    if (!init(env, cls, "jcuda/jnpp/types/Npp64sc")) return JNI_ERR;
    if (!init(env, cls, Npp64sc_re, "re", "J")) return JNI_ERR;
    if (!init(env, cls, Npp64sc_im, "im", "J")) return JNI_ERR;

    // Obtain the fieldIDs for Npp32fc
    if (!init(env, cls, "jcuda/jnpp/types/Npp32fc")) return JNI_ERR;
    if (!init(env, cls, Npp32fc_re, "re", "F")) return JNI_ERR;
    if (!init(env, cls, Npp32fc_im, "im", "F")) return JNI_ERR;

    // Obtain the fieldIDs for Npp64fc
    if (!init(env, cls, "jcuda/jnpp/types/Npp64fc")) return JNI_ERR;
    if (!init(env, cls, Npp64fc_re, "re", "D")) return JNI_ERR;
    if (!init(env, cls, Npp64fc_im, "im", "D")) return JNI_ERR;

    return JNI_VERSION_1_4;

}


/**
 * Create a new TypedPointer object with the given native pointer value
 */
jobject createPointer(JNIEnv *env, jlong pointerValue)
{
    jobject result = env->NewObject(TypedPointer_Class, TypedPointer_Constructor);
    if (env->ExceptionCheck())
    {
        return NULL;
    }
    setPointer(env, result, pointerValue);
    return result;
}


/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, NppiPoint &output)
{
    output.x = (int)env->GetIntField(input, NppiPoint_x);
    output.y = (int)env->GetIntField(input, NppiPoint_y);
    return true;
}

/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, NppiSize &output)
{
    output.width  = (int)env->GetIntField(input, NppiSize_width);
    output.height = (int)env->GetIntField(input, NppiSize_height);
    return true;
}

/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, NppiRect &output)
{
    output.x      = (int)env->GetIntField(input, NppiRect_x);
    output.y      = (int)env->GetIntField(input, NppiRect_y);
    output.width  = (int)env->GetIntField(input, NppiRect_width);
    output.height = (int)env->GetIntField(input, NppiRect_height);
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, NppiDecodeHuffmanSpec* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
		output[i] = (NppiDecodeHuffmanSpec*)getNativePointerValue(env, inputElement);
    }
    return true;
}


/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, NppiEncodeHuffmanSpec* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
		output[i] = (NppiEncodeHuffmanSpec*)getNativePointerValue(env, inputElement);
    }
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, NppiSize output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
		if (!initNative(env, inputElement, output[i]))
		{
			return false;
		}
    }
    return true;
}



