/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2014 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include "JNpp_common.hpp"

// TODO: Many of these methods have been auto-generated and could 
// be replaced by more generic ones 

//=== 2D prmitive arrays =====================================================

/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, double output[4][2])
{
    int outerSize = 4;
    int innerSize = 2;
    jsize size = env->GetArrayLength(input);
    if (size < outerSize)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<outerSize; i++)
    {
        jdoubleArray inputElement = (jdoubleArray)env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException", "Array contains 'null' elements");
            return false;
        }
        jsize elementSize = env->GetArrayLength(inputElement);
        if (elementSize < innerSize)
        {
            ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
            return false;
        }
        jdouble *arrayContents = (jdouble*)env->GetPrimitiveArrayCritical(inputElement, NULL);
        for (int j=0; j<innerSize; j++)
        {
            output[i][j] = (double)arrayContents[j];
        }
        env->ReleasePrimitiveArrayCritical(inputElement, arrayContents, JNI_ABORT);
    }
    return true;
}

/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, double output[4][2], jobjectArray input)
{
    int outerSize = 4;
    int innerSize = 2;
    jsize size = env->GetArrayLength(input);
    if (size < outerSize)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<outerSize; i++)
    {
        jdoubleArray inputElement = (jdoubleArray)env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException", "Array contains 'null' elements");
            return false;
        }
        jsize elementSize = env->GetArrayLength(inputElement);
        if (elementSize < innerSize)
        {
            ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
            return false;
        }
        jdouble *arrayContents = (jdouble*)env->GetPrimitiveArrayCritical(inputElement, NULL);
        for (int j=0; j<innerSize; j++)
        {
            arrayContents[j] = (jdouble)output[i][j];
        }
        env->ReleasePrimitiveArrayCritical(inputElement, arrayContents, 0);
    }
    return true;
}





/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, double output[2][3])
{
    int outerSize = 2;
    int innerSize = 3;
    jsize size = env->GetArrayLength(input);
    if (size < outerSize)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<outerSize; i++)
    {
        jdoubleArray inputElement = (jdoubleArray)env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException", "Array contains 'null' elements");
            return false;
        }
        jsize elementSize = env->GetArrayLength(inputElement);
        if (elementSize < innerSize)
        {
            ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
            return false;
        }
        jdouble *arrayContents = (jdouble*)env->GetPrimitiveArrayCritical(inputElement, NULL);
        for (int j=0; j<innerSize; j++)
        {
            output[i][j] = (double)arrayContents[j];
        }
        env->ReleasePrimitiveArrayCritical(inputElement, arrayContents, JNI_ABORT);
    }
    return true;
}

/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, double output[2][3], jobjectArray input)
{
    int outerSize = 2;
    int innerSize = 3;
    jsize size = env->GetArrayLength(input);
    if (size < outerSize)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<outerSize; i++)
    {
        jdoubleArray inputElement = (jdoubleArray)env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException", "Array contains 'null' elements");
            return false;
        }
        jsize elementSize = env->GetArrayLength(inputElement);
        if (elementSize < innerSize)
        {
            ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
            return false;
        }
        jdouble *arrayContents = (jdouble*)env->GetPrimitiveArrayCritical(inputElement, NULL);
        for (int j=0; j<innerSize; j++)
        {
            arrayContents[j] = (jdouble)output[i][j];
        }
        env->ReleasePrimitiveArrayCritical(inputElement, arrayContents, 0);
    }
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp32f output[3][4])
{
    int outerSize = 3;
    int innerSize = 4;
    jsize size = env->GetArrayLength(input);
    if (size < outerSize)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<outerSize; i++)
    {
        jfloatArray inputElement = (jfloatArray)env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException", "Array contains 'null' elements");
            return false;
        }
        jsize elementSize = env->GetArrayLength(inputElement);
        if (elementSize < innerSize)
        {
            ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
            return false;
        }
        jfloat *arrayContents = (jfloat*)env->GetPrimitiveArrayCritical(inputElement, NULL);
        for (int j=0; j<innerSize; j++)
        {
            output[i][j] = (Npp32f)arrayContents[j];
        }
        env->ReleasePrimitiveArrayCritical(inputElement, arrayContents, JNI_ABORT);
    }
    return true;
}

/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp32f output[3][4], jobjectArray input)
{
    int outerSize = 3;
    int innerSize = 4;
    jsize size = env->GetArrayLength(input);
    if (size < outerSize)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<outerSize; i++)
    {
        jfloatArray inputElement = (jfloatArray)env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException", "Array contains 'null' elements");
            return false;
        }
        jsize elementSize = env->GetArrayLength(inputElement);
        if (elementSize < innerSize)
        {
            ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
            return false;
        }
        jfloat *arrayContents = (jfloat*)env->GetPrimitiveArrayCritical(inputElement, NULL);
        for (int j=0; j<innerSize; j++)
        {
            arrayContents[j] = (jfloat)output[i][j];
        }
        env->ReleasePrimitiveArrayCritical(inputElement, arrayContents, 0);
    }
    return true;
}


