/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2013 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package jcuda.jnpp;

/**
 * Rounding modes.
 * 
 * The enumerated rounding modes are used by a large number of NPP primitives
 * to allow the user to specify the method by which fractional values are converted
 * to integer values. 
 *
 * For NPP release 5.5 new names for the three rounding modes are introduced that are
 * based on the naming conventions for rounding modes set forth in the IEEE-754
 * floating-point standard. Developers are encouraged to use the new, longer names
 * to be future proof as the legacy names will be deprecated in subsequent NPP releases.
 */
public class NppRoundMode
{
    /** 
     * Round to the nearest even integer.
     * All fractional numbers are rounded to their nearest integer. The ambiguous
     * cases (i.e. \<integer\>.5) are rounded to the closest even integer.
     * E.g.
     * - roundNear(0.5) = 0
     * - roundNear(0.6) = 1
     * - roundNear(1.5) = 2
     * - roundNear(-1.5) = -2
     */
    public static final int NPP_RND_NEAR = 0;
    public static final int NPP_ROUND_NEAREST_TIES_TO_EVEN = NPP_RND_NEAR;

    /** 
     * Round according to financial rule.
     * All fractional numbers are rounded to their nearest integer. The ambiguous
     * cases (i.e. \<integer\>.5) are rounded away from zero.
     * E.g.
     * - roundFinancial(0.4)  = 0
     * - roundFinancial(0.5)  = 1
     * - roundFinancial(-1.5) = -2
     */
    public static final int NPP_RND_FINANCIAL = 1;
    public static final int NPP_ROUND_NEAREST_TIES_AWAY_FROM_ZERO = NPP_RND_FINANCIAL;
    
    /**
     * Round towards zero (truncation). 
     * All fractional numbers of the form \<integer\>.\<decimals\> are truncated to
     * \<integer\>.
     * - roundZero(1.5) = 1
     * - roundZero(1.9) = 1
     * - roundZero(-2.5) = -2
     */
    public static final int NPP_RND_ZERO = 2;
    public static final int NPP_ROUND_TOWARD_ZERO = NPP_RND_ZERO;
    
    /**
     * Returns the String identifying the given NppRoundMode
     *
     * @param m The NppRoundMode
     * @return The String identifying the given NppRoundMode
     */
    public static String stringFor(int m)
    {
        switch (m)
        {
            case NPP_ROUND_NEAREST_TIES_TO_EVEN: return "NPP_RND_NEAR/NPP_ROUND_NEAREST_TIES_TO_EVEN";
            case NPP_ROUND_NEAREST_TIES_AWAY_FROM_ZERO: return "NPP_RND_FINANCIAL/NPP_ROUND_NEAREST_TIES_AWAY_FROM_ZERO";
            case NPP_ROUND_TOWARD_ZERO: return "NPP_RND_ZERO/NPP_ROUND_TOWARD_ZERO";
        }
        return "INVALID NppRoundMode: " + m;
    }

    /**
     * Private constructor to prevent instantiation.
     */
    private NppRoundMode()
    {
    }
    
}
